/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: fwcompiler.h,v 1.10 2001/04/05 03:15:23 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Puplic License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#ifndef _FWCOMPILER_H_
#define _FWCOMPILER_H_

#include <parser.h>
#include <tree.h>

#include <glib.h>

#define FROMXMLCAST(x) ((const char *)x)
#define TOXMLCAST(x) ((xmlChar *)x)

/*
 *   Our standard IDs :
 */
#define    AnyNetworkId    "sysid0"
#define    AnyIPServiceId  "sysid1"
#define    AnyIntervalId   "sysid2"
    		    
#define    ObjectsId       "stdid01"
#define    HostsId         "stdid02"
#define    NetworksId      "stdid03"
#define    ObjectGroupsId  "stdid04"
#define    ServicesId      "stdid05"
#define    IPServicesId    "stdid06"
#define    ICMPServicesId  "stdid07"
#define    UDPServicesId   "stdid08"
#define    TCPServicesId   "stdid09"
#define    ServiceGroupsId "stdid10"
#define    TimeId          "stdid11"
#define    FirewallsId     "stdid12"


typedef struct {

    int   num;
    const char* direction;
    const char* action;
    int   log;

    xmlNodePtr interface;
    
    xmlNodePtr src;
    xmlNodePtr dst;
    xmlNodePtr srv;

    xmlNodePtr options;
    
    char  *chain;
    
}  policy_rule;

typedef struct {

    int   num;

    xmlNodePtr o_src;
    xmlNodePtr o_dst;
    xmlNodePtr o_srv;
    
    xmlNodePtr t_src;
    xmlNodePtr t_dst;
    xmlNodePtr t_srv;

    xmlNodePtr options;
    
}  nat_rule;

extern xmlNodePtr AnyNetwork;

extern char*  getNewTemporaryChain();

extern int  cmpPolicyRules(policy_rule *r1,policy_rule *r2);
extern int  cmpNatRules(nat_rule *r1,nat_rule *r2);


#define  getXmlNodeContent(node) \
     FROMXMLCAST( xmlNodeGetContent(node) )
/*
extern char*      getXmlAttribute(xmlNodePtr node,const char* attr_name);
extern char*      getXmlNodeContent(xmlNodePtr node);
*/
extern xmlNodePtr  getFirstChildXmlNode(xmlNodePtr node,
				       const char* child_element_name);

extern const char*       getXmlElementName(xmlNodePtr node);

extern int               isElement(xmlNodePtr node,const char* element_name);
extern int               isReference(xmlNodePtr node);
extern int               isGroup(xmlNodePtr node);
extern int               isAny(xmlNodePtr node);
extern int               isAnyNetwork(xmlNodePtr node);
extern int               isAnyService(xmlNodePtr node);

extern const char*       getId(xmlNodePtr node);
extern const char*       getName(xmlNodePtr node);
extern const char*       getComment(xmlNodePtr node);

extern const char*       getRef(xmlNodePtr node);

extern int         getBool(xmlNodePtr node, const char* attribute);
extern int         getInt(xmlNodePtr node, const char* attribute);
extern const char* getStr(xmlNodePtr node, const char* attribute);

extern xmlNodePtr  getById(const char* id);

extern xmlNodePtr  resolveReference(xmlNodePtr node);

extern const char* getOptionStr(xmlNodePtr option_object,const char* option_name);
extern int getOptionInt(xmlNodePtr option_object,const char* option_name);
extern int getOptionBool(xmlNodePtr option_object,const char* option_name);

extern int cmpObjects(xmlNodePtr obj1,xmlNodePtr obj2);

/************************************************************************
 *  Implement these functions to produce compiler for new firewall
 *  platform.
 */

extern void prologue(xmlNodePtr firewall,xmlNodePtr fw_options);
extern void epilogue(xmlNodePtr firewall,xmlNodePtr fw_options);


/*
 *    Process policy rule. This function is platform-dependent and
 *    should be reimplemented for new firewall platform.
 * 
 *    Each rule defined in GUI will be converted to a group of
 *    subrules if groups or multiple objects are used as rule
 *    elements. Each individual subrule then will be passed to
 *    processPolicyRule, thus guaranteeing that this function has to
 *    deal with exactly one undividable object as source, destination
 *    and service.
 *
 *    Arguments:
 *
 *    xmlNodePtr firewall      - XML node for firewall object
 *    int  subrule_no          - number of subrule 
 *    int  num_subrules        - total number of subrules
 *    policy_rule* rule_line   - structure describes subrule
 *
 * */

extern void processInterfacePolicyRule(xmlNodePtr firewall,
				       int        num,
				       const char *comment,
				       int        src_neg,
				       int        dst_neg,
				       GSList    *subrules);

extern void processPolicyRule(xmlNodePtr firewall,
			      int        num,
			      const char *comment,
			      int        src_neg,
			      int        dst_neg,
			      int        srv_neg,
			      GSList    *subrules);

/*
 *   NAT rules are processed in the same way as policy
 *   rules. Arguments here have the same meaning as above
 */

extern void processNATRule(xmlNodePtr firewall,
			   int        num,
			   nat_rule* rule_line);


#endif







